@@ -19,6 +19,7 @@
 # along with duplicity; if not, write to the Free Software Foundation,
 # Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 
+import os
 import time
 
 import duplicity.backend
@@ -29,7 +30,90 @@ from duplicity.util import exception_traceback
 from duplicity.backend import retry
 from duplicity import progress
 
-BOTO_MIN_VERSION = "1.6a"
+BOTO_MIN_VERSION = "2.1.1"
+
+
+def get_connection(scheme, parsed_url, storage_uri):
+    try:
+        from boto.s3.connection import S3Connection
+        assert hasattr(S3Connection, 'lookup')
+
+        # Newer versions of boto default to using
+        # virtual hosting for buckets as a result of
+        # upstream deprecation of the old-style access
+        # method by Amazon S3. This change is not
+        # backwards compatible (in particular with
+        # respect to upper case characters in bucket
+        # names); so we default to forcing use of the
+        # old-style method unless the user has
+        # explicitly asked us to use new-style bucket
+        # access.
+        #
+        # Note that if the user wants to use new-style
+        # buckets, we use the subdomain calling form
+        # rather than given the option of both
+        # subdomain and vhost. The reason being that
+        # anything addressable as a vhost, is also
+        # addressable as a subdomain. Seeing as the
+        # latter is mostly a convenience method of
+        # allowing browse:able content semi-invisibly
+        # being hosted on S3, the former format makes
+        # a lot more sense for us to use - being
+        # explicit about what is happening (the fact
+        # that we are talking to S3 servers).
+
+        try:
+            from boto.s3.connection import OrdinaryCallingFormat
+            from boto.s3.connection import SubdomainCallingFormat
+            cfs_supported = True
+            calling_format = OrdinaryCallingFormat()
+        except ImportError:
+            cfs_supported = False
+            calling_format = None
+
+        if globals.s3_use_new_style:
+            if cfs_supported:
+                calling_format = SubdomainCallingFormat()
+            else:
+                log.FatalError("Use of new-style (subdomain) S3 bucket addressing was"
+                               "requested, but does not seem to be supported by the "
+                               "boto library. Either you need to upgrade your boto "
+                               "library or duplicity has failed to correctly detect "
+                               "the appropriate support.",
+                               log.ErrorCode.boto_old_style)
+        else:
+            if cfs_supported:
+                calling_format = OrdinaryCallingFormat()
+            else:
+                calling_format = None
+
+    except ImportError:
+        log.FatalError("This backend (s3) requires boto library, version %s or later, "
+                       "(http://code.google.com/p/boto/)." % BOTO_MIN_VERSION,
+                       log.ErrorCode.boto_lib_too_old)
+
+    if not parsed_url.hostname:
+        # Use the default host.
+        conn = storage_uri.connect(is_secure=(not globals.s3_unencrypted_connection))
+    else:
+        assert scheme == 's3'
+        conn = storage_uri.connect(host=parsed_url.hostname,
+                                   is_secure=(not globals.s3_unencrypted_connection))
+
+    if hasattr(conn, 'calling_format'):
+        if calling_format is None:
+            log.FatalError("It seems we previously failed to detect support for calling "
+                           "formats in the boto library, yet the support is there. This is "
+                           "almost certainly a duplicity bug.",
+                           log.ErrorCode.boto_calling_format)
+        else:
+            conn.calling_format = calling_format
+
+    else:
+        # Duplicity hangs if boto gets a null bucket name.
+        # HC: Caught a socket error, trying to recover
+        raise BackendException('Boto requires a bucket name.')
+    return conn
 
 
 class BotoBackend(duplicity.backend.Backend):
@@ -45,9 +129,9 @@ class BotoBackend(duplicity.backend.Backend):
     """
 
     def __init__(self, parsed_url):
+        import boto
         duplicity.backend.Backend.__init__(self, parsed_url)
 
-        import boto
         assert boto.Version >= BOTO_MIN_VERSION
 
         # This folds the null prefix and all null parts, which means that:
@@ -76,96 +160,29 @@ class BotoBackend(duplicity.backend.Backend):
         # boto uses scheme://bucket[/name] and specifies hostname on connect()
         self.boto_uri_str = '://'.join((parsed_url.scheme[:2],
                                         parsed_url.path.lstrip('/')))
-        self.storage_uri = boto.storage_uri(self.boto_uri_str)
         self.resetConnection()
+        self._listed_keys = {}
 
-    def resetConnection(self):
+    def close(self):
+        del self._listed_keys
+        self._listed_keys = {}
         self.bucket = None
         self.conn = None
+        self.storage_uri = None
+        del self.conn
+        del self.storage_uri
 
-        try:
-            from boto.s3.connection import S3Connection
-            from boto.s3.key import Key
-            assert hasattr(S3Connection, 'lookup')
-
-            # Newer versions of boto default to using
-            # virtual hosting for buckets as a result of
-            # upstream deprecation of the old-style access
-            # method by Amazon S3. This change is not
-            # backwards compatible (in particular with
-            # respect to upper case characters in bucket
-            # names); so we default to forcing use of the
-            # old-style method unless the user has
-            # explicitly asked us to use new-style bucket
-            # access.
-            #
-            # Note that if the user wants to use new-style
-            # buckets, we use the subdomain calling form
-            # rather than given the option of both
-            # subdomain and vhost. The reason being that
-            # anything addressable as a vhost, is also
-            # addressable as a subdomain. Seeing as the
-            # latter is mostly a convenience method of
-            # allowing browse:able content semi-invisibly
-            # being hosted on S3, the former format makes
-            # a lot more sense for us to use - being
-            # explicit about what is happening (the fact
-            # that we are talking to S3 servers).
-
-            try:
-                from boto.s3.connection import OrdinaryCallingFormat
-                from boto.s3.connection import SubdomainCallingFormat
-                cfs_supported = True
-                calling_format = OrdinaryCallingFormat()
-            except ImportError:
-                cfs_supported = False
-                calling_format = None
-
-            if globals.s3_use_new_style:
-                if cfs_supported:
-                    calling_format = SubdomainCallingFormat()
-                else:
-                    log.FatalError("Use of new-style (subdomain) S3 bucket addressing was"
-                                   "requested, but does not seem to be supported by the "
-                                   "boto library. Either you need to upgrade your boto "
-                                   "library or duplicity has failed to correctly detect "
-                                   "the appropriate support.",
-                                   log.ErrorCode.boto_old_style)
-            else:
-                if cfs_supported:
-                    calling_format = OrdinaryCallingFormat()
-                else:
-                    calling_format = None
-
-        except ImportError:
-            log.FatalError("This backend (s3) requires boto library, version %s or later, "
-                           "(http://code.google.com/p/boto/)." % BOTO_MIN_VERSION,
-                           log.ErrorCode.boto_lib_too_old)
-
-        if not self.parsed_url.hostname:
-            # Use the default host.
-            self.conn = self.storage_uri.connect(
-                is_secure=(not globals.s3_unencrypted_connection))
-        else:
-            assert self.scheme == 's3'
-            self.conn = self.storage_uri.connect(
-                host=self.parsed_url.hostname,
-                is_secure=(not globals.s3_unencrypted_connection))
-
-        if hasattr(self.conn, 'calling_format'):
-            if calling_format is None:
-                log.FatalError("It seems we previously failed to detect support for calling "
-                               "formats in the boto library, yet the support is there. This is "
-                               "almost certainly a duplicity bug.",
-                               log.ErrorCode.boto_calling_format)
-            else:
-                self.conn.calling_format = calling_format
-
-        else:
-            # Duplicity hangs if boto gets a null bucket name.
-            # HC: Caught a socket error, trying to recover
-            raise BackendException('Boto requires a bucket name.')
-
+    def resetConnection(self):
+        import boto
+        if getattr(self, 'conn', False):
+            self.conn.close()
+        self.bucket = None
+        self.conn = None
+        self.storage_uri = None
+        del self.conn
+        del self.storage_uri
+        self.storage_uri = boto.storage_uri(self.boto_uri_str)
+        self.conn = get_connection(self.scheme, self.parsed_url, self.storage_uri)
         self.bucket = self.conn.lookup(self.bucket_name)
 
     def put(self, source_path, remote_filename=None):
@@ -181,6 +198,7 @@ class BotoBackend(duplicity.backend.Backend):
                 break
             if n > 1:
                 time.sleep(30)
+                self.resetConnection()
             try:
                 try:
                     self.bucket = self.conn.get_bucket(self.bucket_name, validate=True)
@@ -198,12 +216,10 @@ class BotoBackend(duplicity.backend.Backend):
                          "" % (n, self.bucket_name,
                                e.__class__.__name__,
                                str(e)))
-                self.resetConnection()
 
         if not remote_filename:
             remote_filename = source_path.get_filename()
-        key = self.storage_uri.new_key()
-        key.key = self.key_prefix + remote_filename
+        key = self.bucket.new_key(self.key_prefix + remote_filename)
 
         for n in range(1, globals.num_retries+1):
             if n > 1:
@@ -216,14 +232,25 @@ class BotoBackend(duplicity.backend.Backend):
                 storage_class = 'STANDARD'
             log.Info("Uploading %s/%s to %s Storage" % (self.straight_url, remote_filename, storage_class))
             try:
-                key.set_contents_from_filename(source_path.name, {'Content-Type': 'application/octet-stream',
-                                                                  'x-amz-storage-class': storage_class},
-                                                cb=progress.report_transfer,
-                                                num_cb=(max(2, 8 * globals.volsize / (1024 * 1024)))
-                                              ) # Max num of callbacks = 8 times x megabyte
-
-                key.close()
+                if globals.s3_use_sse:
+                    headers = {
+                    'Content-Type': 'application/octet-stream',
+                    'x-amz-storage-class': storage_class,
+                    'x-amz-server-side-encryption': 'AES256'
+                }
+                else:
+                    headers = {
+                    'Content-Type': 'application/octet-stream',
+                    'x-amz-storage-class': storage_class
+                }
+                
+                upload_start = time.time()
+                self.upload(source_path.name, key, headers)
+                upload_end = time.time()
+                total_s = abs(upload_end-upload_start) or 1  # prevent a zero value!
+                rough_upload_speed = os.path.getsize(source_path.name)/total_s
                 self.resetConnection()
+                log.Debug("Uploaded %s/%s to %s Storage at roughly %f bytes/second" % (self.straight_url, remote_filename, storage_class, rough_upload_speed))
                 return
             except Exception, e:
                 log.Warn("Upload '%s/%s' failed (attempt #%d, reason: %s: %s)"
@@ -239,17 +266,18 @@ class BotoBackend(duplicity.backend.Backend):
         raise BackendException("Error uploading %s/%s" % (self.straight_url, remote_filename))
 
     def get(self, remote_filename, local_path):
-        key = self.storage_uri.new_key()
-        key.key = self.key_prefix + remote_filename
+        key_name = self.key_prefix + remote_filename
+        self.pre_process_download(remote_filename, wait=True)
+        key = self._listed_keys[key_name]
         for n in range(1, globals.num_retries+1):
             if n > 1:
                 # sleep before retry (new connection to a **hopeful** new host, so no need to wait so long)
                 time.sleep(10)
             log.Info("Downloading %s/%s" % (self.straight_url, remote_filename))
             try:
+                self.resetConnection()
                 key.get_contents_to_filename(local_path.name)
                 local_path.setdata()
-                self.resetConnection()
                 return
             except Exception, e:
                 log.Warn("Download %s/%s failed (attempt #%d, reason: %s: %s)"
@@ -259,19 +287,20 @@ class BotoBackend(duplicity.backend.Backend):
                                e.__class__.__name__,
                                str(e)), 1)
                 log.Debug("Backtrace of previous error: %s" % (exception_traceback(),))
-                self.resetConnection()
+
         log.Warn("Giving up trying to download %s/%s after %d attempts" %
                 (self.straight_url, remote_filename, globals.num_retries))
         raise BackendException("Error downloading %s/%s" % (self.straight_url, remote_filename))
 
-    def list(self):
+    def _list(self):
         if not self.bucket:
-            return []
+            raise BackendException("No connection to backend")
 
         for n in range(1, globals.num_retries+1):
             if n > 1:
                 # sleep before retry
                 time.sleep(30)
+                self.resetConnection()
             log.Info("Listing %s" % self.straight_url)
             try:
                 return self._list_filenames_in_bucket()
@@ -297,10 +326,11 @@ class BotoBackend(duplicity.backend.Backend):
         # Because of the need for this optimization, it should be left as is.
         #for k in self.bucket.list(prefix = self.key_prefix + 'd', delimiter = '/'):
         filename_list = []
-        for k in self.bucket.list(prefix = self.key_prefix, delimiter = '/'):
+        for k in self.bucket.list(prefix=self.key_prefix, delimiter='/'):
             try:
                 filename = k.key.replace(self.key_prefix, '', 1)
                 filename_list.append(filename)
+                self._listed_keys[k.key] = k
                 log.Debug("Listed %s/%s" % (self.straight_url, filename))
             except AttributeError:
                 pass
@@ -329,6 +359,53 @@ class BotoBackend(duplicity.backend.Backend):
             else:
                 return {'size': None}
 
-duplicity.backend.register_backend("gs", BotoBackend)
-duplicity.backend.register_backend("s3", BotoBackend)
-duplicity.backend.register_backend("s3+http", BotoBackend)
+    def upload(self, filename, key, headers):
+            key.set_contents_from_filename(filename, headers,
+                                           cb=progress.report_transfer,
+                                           num_cb=(max(2, 8 * globals.volsize / (1024 * 1024)))
+                                           )  # Max num of callbacks = 8 times x megabyte
+            key.close()
+
+    def pre_process_download(self, files_to_download, wait=False):
+        # Used primarily to move files in Glacier to S3
+        if isinstance(files_to_download, basestring):
+            files_to_download = [files_to_download]
+
+        for remote_filename in files_to_download:
+            success = False
+            for n in range(1, globals.num_retries+1):
+                if n > 1:
+                    # sleep before retry (new connection to a **hopeful** new host, so no need to wait so long)
+                    time.sleep(10)
+                    self.resetConnection()
+                try:
+                    key_name = self.key_prefix + remote_filename
+                    if not self._listed_keys.get(key_name, False):
+                        self._listed_keys[key_name] = list(self.bucket.list(key_name))[0]
+                    key = self._listed_keys[key_name]
+
+                    if key.storage_class == "GLACIER":
+                        # We need to move the file out of glacier
+                        if not self.bucket.get_key(key.key).ongoing_restore:
+                            log.Info("File %s is in Glacier storage, restoring to S3" % remote_filename)
+                            key.restore(days=1)  # Shouldn't need this again after 1 day
+                        if wait:
+                            log.Info("Waiting for file %s to restore from Glacier" % remote_filename)
+                            while self.bucket.get_key(key.key).ongoing_restore:
+                                time.sleep(60)
+                                self.resetConnection()
+                            log.Info("File %s was successfully restored from Glacier" % remote_filename)
+                    success = True
+                    break
+                except Exception, e:
+                    log.Warn("Restoration from Glacier for file %s/%s failed (attempt #%d, reason: %s: %s)"
+                             "" % (self.straight_url,
+                                   remote_filename,
+                                   n,
+                                   e.__class__.__name__,
+                                   str(e)), 1)
+                    log.Debug("Backtrace of previous error: %s" % (exception_traceback(),))
+            if not success:
+                log.Warn("Giving up trying to restore %s/%s after %d attempts" %
+                        (self.straight_url, remote_filename, globals.num_retries))
+                raise BackendException("Error restoring %s/%s from Glacier to S3" % (self.straight_url, remote_filename))
