@@ -26,11 +26,20 @@ import socket
 
 
 # The current version of duplicity
-version = "0.6.23"
+version = "0.6.25"
 
-# Default file_prefix value
+# Prefix for all files (appended before type-specific prefixes)
 file_prefix = ""
 
+# Prefix for manifest files only
+file_prefix_manifest = ""
+
+# Prefix for archive files only
+file_prefix_archive = ""
+
+# Prefix for sig files only
+file_prefix_signature = ""
+
 # The name of the current host, or None if it cannot be set
 hostname = socket.getfqdn()
 
@@ -191,6 +200,15 @@ s3_multipart_chunk_size = 25 * 1024 * 1024
 # Minimum chunk size accepted by S3
 s3_multipart_minimum_chunk_size = 5 * 1024 * 1024
 
+# Maximum number of processes to use while doing a multipart upload to S3
+s3_multipart_max_procs = None
+
+# Maximum time to wait for a part to finish when doig a multipart upload to S3
+s3_multipart_max_timeout = None
+
+# Use server side encryption in s3
+s3_use_sse = False
+
 # Whether to use the full email address as the user name when
 # logging into an imap server. If false just the user name
 # part of the email address is used.
@@ -263,3 +281,9 @@ progress = False
 # Controls the upload progress messages refresh rate. Default: update each
 # 3 seconds
 progress_rate = 3
+
+# Level of Redundancy in % for Par2 files
+par2_redundancy = 10
+
+# Verbatim par2 other options
+par2_options = ""
